/*
 * This file is a part of PikoTools
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2016, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *    
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *    
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef headerfile_picotools_mainspaceparser_mainparser
#define headerfile_picotools_mainspaceparser_mainparser

#include "space/space.h"
#include <string>
#include <vector>


namespace PT
{


/*
	a very little parser for main(int argc, char ** argv) parameters
	look in sample/sample.cpp how to use the parser
*/
class MainSpaceParser
{
public:

	MainSpaceParser();
	~MainSpaceParser();

	enum Status
	{
		status_ok = 0,
		status_space_not_assigned = 1,
		status_syntax_error = 2,
		status_reading_eof = 3 /* CHANGE ME give a better name */
	};

	void SetSpace(Space & space);
	Status Parse(int argc, const char ** argv);

	void UTF8(bool utf8);

	std::wstring & GetErrorToken();

private:

	Space * space;
	Space * options_space;
	std::wstring wide_arg, temp_arg, temp_val;
	std::vector<std::wstring> temp_list_val;
	bool use_utf8;
	Status last_status;
	std::wstring last_error_token;


	void ConvertStr(const char * src, std::wstring & dst);
	void Parse(size_t argc, const char ** argv, size_t & argv_index);
	void ParseSingleArgument(size_t argc, const char ** argv, size_t & argv_index);
	void ParseMultiArgument(size_t argc, const char ** argv, size_t & argv_index);
	size_t RequireOption(const std::wstring & arg);
	void AddValueToItem(const std::wstring & name, const std::wstring & empty_value, const std::vector<std::wstring> & list);

};


} // namespace


#endif
