/*
 * This file is a part of PikoTools
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2012, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *    
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *    
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "spacetojson.h"


namespace PT
{


void SpaceToJSON::Clear()
{
	numeric.clear();
	boolean.clear();
	table.clear();
}


void SpaceToJSON::TreatAsTable(const wchar_t * space_name)
{
	table.insert(space_name);
}


void SpaceToJSON::TreatAsTable(const std::wstring & space_name)
{
	table.insert(space_name);
}



void SpaceToJSON::TreatAsNumeric(const wchar_t * name)
{
	numeric.insert(name);
}


void SpaceToJSON::TreatAsNumeric(const std::wstring & name)
{
	numeric.insert(name);

}


void SpaceToJSON::TreatAsBool(const wchar_t * name)
{
	boolean.insert(name);
}

void SpaceToJSON::TreatAsBool(const std::wstring & name)
{
	boolean.insert(name);
}



bool SpaceToJSON::IsNumeric(const std::wstring & name)
{
	std::set<std::wstring>::iterator i = numeric.find(name);
	return i != numeric.end();
}


bool SpaceToJSON::IsBool(const std::wstring & name)
{
	std::set<std::wstring>::iterator i = boolean.find(name);
	return i != boolean.end();
}

bool SpaceToJSON::IsTable(const std::wstring & name)
{
	std::set<std::wstring>::iterator i = table.find(name);
	return i != table.end();
}





} // namespace

